<?php
/**
 * @version     0.0.1
 * @package     Babel-U-Lib
 * @copyright   Copyright (C) 2011. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author     Mathew Lenning - http://mathewlenning.com/
 */
// No direct access
defined('_JEXEC') or die;

class Babelu_libControllerJoomlaDispatcher extends Babelu_libControllerJoomlaBase
{	
	/**
	 * The controller for this task
	 * @var Babelu_libControllerJoomlaBase
	 */
	protected $controller;

	/**
	 * Uses the input to select the task controller and set the subject to the input
	 * uses the 'option' and 'task' to determin $this->controller
	 * format Option.'Controller'.Task
	 * Also sets the subject using the task
	 * if the task isn't in controller.subject format, it defaults to task.view
	 * if no task is set, it defaults to display.view
	 * if the task and the view are not set, it defaults to display.default
	 * @param JInput $input
	 * @param JApplicationBase $app
	 * @throws InvalidArgumentException
	 */
	public function __construct(JInput $input, $app = null, $config = array())
	{	
		$command = $input->get('task', 'display', 'cmd');
		
		if (!strpos($command, '.'))
		{
			if (!array_key_exists('default_view', $config))
			{
				$config['default_view'] = 'default';
			}
			
			$command .= '.'.$input->get('view',$config['default_view']);
		}
		
		list($controllerName, $subject) = explode('.', $command);
		
		$config['task'] = $controllerName;
		$config['subject'] = $subject;
		
		parent::__construct($input, $app, $config);
		
		$optionPrefix = $this->getPrefix();
		
		$class = $optionPrefix.'Controller'.ucfirst($controllerName);

		if (!class_exists($class)) // get the fallback
		{
			$class = $this->getFallbackClassName($controllerName);
			
			if (!class_exists($class)) // PANIC!
			{
				$format   = $input->getWord('format', 'html');
				throw new InvalidArgumentException(JText::sprintf('JLIB_APPLICATION_ERROR_INVALID_CONTROLLER', $controllerName, $format));
			}
		}
		
		$input->set('task', $controllerName);

		$this->controller = new $class($input, $app, $config);
	}
	
	
	protected function getFallbackClassName($controllerName)
	{
		$class_name = 'Babelu_libControllerJoomla'.ucfirst($controllerName);
		return $class_name;
	}
	
	public function execute()
	{
		if ($this->controller->execute())
		{
			return true;
		}
		return false;
	}
	
	/**
	 * (non-PHPdoc)
	 * @see Babelu_libControllerJoomlaBase::redirect()
	 */
	public function redirect()
	{
		return $this->controller->redirect();
	}
}