<?php
/**
 * @version     0.0.1
 * @package     Babel-U-Lib
 * @copyright   Copyright (C) 2011. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author     Mathew Lenning - http://mathewlenning.com/
 */
// No direct access
defined('_JEXEC') or die;

abstract class Babelu_libControllerJoomlaPdfBase extends Babelu_libControllerJoomlaDisplay
{
	/**
	 * Associative Array pdf objects
	 * @var array of HTML2PDF
	 */
	protected $pdfs = array();

	/**
	 * Method to get a pdf object
	 * @param string $orientation L = Landscape, P = Portrait
	 * @param string $format A4, A5, or LETTER
	 * @param string $language two letter codes I.E. en = English
	 * @return HTML2PDF
	 */
	public function getPdfObject($key, $orientation, $format,  $language)
	{
		if (!array_key_exists($key, $this->pdfs))
		{
			include_once JPATH_SITE.'/media/babelu_lib/html2pdf_v4.03/html2pdf.class.php';
			
			$orientation = strtoupper($orientation);
			$format = strtoupper($format);
			$language = strtolower($language);
		
			$this->pdfs[$key] = new HTML2PDF($orientation, $format, $language);
		}
		
		return $this->pdfs[$key];
	}
	
	public function writeHTML($key, $html, $debug = false)
	{
		if (array_key_exists($key, $this->pdfs))
		{	
			$result = $this->pdfs[$key]->writeHTML($html, $debug);
			
			if ($debug)
			{
				return $result;
			}
			
			return true;
		}
		return false;
	}
	
	/**
	 * Send the document to a given destination: string, local file or browser.
	 * Dest can be :
	 *  I : send the file inline to the browser (default). The plug-in is used if available. The name given by name is used when one selects the "Save as" option on the link generating the PDF.
	 *  D : send to the browser and force a file download with the name given by name.
	 *  F : save to a local server file with the name given by name.
	 *  S : return the document as a string. name is ignored.
	 *  FI: equivalent to F + I option
	 *  FD: equivalent to F + D option
	 *  true  => I
	 *  false => S
	 *
	 * @param  string $key  pdfs array index key
	 * @param  string $filename The name of the file when saved.
	 * @param  string $destination Destination where to send the document.
	 * @return string content of the PDF, if $dest=S
	 * @see HTML2PDF::Output
	 * @access public
	
	 */
	public function getOutput($key, $filename = '', $destination = false)
	{
		if (array_key_exists($key, $this->pdfs))
		{
			return $this->pdfs[$key]->Output($filename, $destination);
		}
		
		return false;
	}
	
}
