<?php
/**
 * @version     0.0.1
 * @package     Babel-U-Lib
 * @copyright   Copyright (C) 2011. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author     Mathew Lenning - http://mathewlenning.com/
 */
// No direct access
defined('_JEXEC') or die;

abstract class Babelu_libModelJoomlaItem extends Babelu_libModelJoomlaData
{
	/**
	 * Item prepared for PDF display
	 * @var JObject
	 */
	protected $pdfItem;
	
	/**
	 * Method to get a single record.
	 *
	 * @param   integer  $pk  The id of the primary key.
	 *
	 * @return  mixed    Object on success, false on failure.
	 *
	 * @since   12.2
	 */
	public function getItem($pk = null)
	{
		$config = $this->config;
	
		if(empty($pk))
		{
			$context= $this->getContext();
			$pk = (int) $this->getState($context. '.id');
		}
	
		$table = $this->getTable();
	
		$table->load($pk);
	
	
		// Convert to the JObject before adding other data.
		$properties = $table->getProperties(1);
		$item = JArrayHelper::toObject($properties, 'JObject');
	
		if (property_exists($item, 'params'))
		{
			$registry = new JRegistry;
			$registry->loadString($item->params);
			$item->params = $registry->toArray();
		}
	
		return $item;
	}
	
	/**
	 * Method to get an item for pdf consumption.
	 * This method insures that the returned item has the required
	 * orientation, format, and language properties.
	 *
	 * @param   integer  $pk  The id of the primary key.
	 *
	 * @return  mixed    JObject on success, false on failure.
	 *
	 * @see Babelu_libModelJoomlaAdmin::getItem
	 */
	public function getPdfItem($pk = null)
	{
		if (!is_object($this->pdfItem))
		{
			$item = $this->getItem($pk);
	
			if (!is_object($item))
			{
				return false;
			}
				
			$this->pdfItem = $this->appendPdfDocumentSettings($item);;
		}
	
		return $this->pdfItem;
	}
	
	/**
	 * Method to append format, orientation, and language to item
	 * @param JObject $item
	 * @return JObject $item
	 */
	protected function appendPdfDocumentSettings($item)
	{
		$config = $this->config;
	
		$params = JComponentHelper::getParams($config['option']);
	
		if (!property_exists($item, 'format'))
		{
			$item->format = $params->get('pdfFormat', 'A4');
		}
	
		if (!property_exists($item, 'orientation'))
		{
			$item->orientation = $params->get('pdfOrientation', 'P');
		}
	
		if (!property_exists($item, 'language'))
		{
			$item->language = $params->get('pdfLanguage', $this->getDefaultLanguage());
		}
	
		return $item;
	}
	
	/**
	 * Method to get the default two letter language code
	 * @return string example en, fr, es
	 */
	protected function getDefaultLanguage()
	{
		$language = JFactory::getLanguage();
		$lang = substr($language->getDefault(), 0, 2);
		return $lang;
	}
}
