<?php
/**
 * @version     0.0.1
 * @package     Babel-U-Lib
 * @copyright   Copyright (C) 2011. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author     Mathew Lenning - http://mathewlenning.com/
 */
// No direct access
defined('_JEXEC') or die;

class Babelu_libUtilityJoomlaToolbar
{
	public function getButtons($config)
	{
		$className = ucfirst(substr($config['option'], 4)).'View'.ucfirst($config['subject']).ucfirst($config['layout']).'buttons';
		$fallBackName = 'Babelu_libViewButtons'.ucfirst($config['layout']);
		
		if (class_exists($className))
		{
			$instance = new $className();
			return $instance->getButtons($config);
		}
		elseif (class_exists($fallBackName))
		{
			$instance = new $fallBackName();
			return $instance->getButtons($config);
		}
		else
		{
			return array();
		}
	}	
	
	/**
	 * Method to build the toolbar for the view.
	 * 
	 * {@link JToolBar::appendButton}
	 * @param string $assetName used for ACL 
	 * @param string $header untranslated translation string
	 * @param string $icon The space-separated names of the image.
	 * @param array $buttons  array of arrays array('type' => $type, 'task' => subject, 'alt' => $alt, 'actions' => array(of actions), $msg = 'MESSAGE_STRING')
	 */
	public function addToolBar($assetName, $header, $icon,  $buttons = array())
	{
		JToolBarHelper::title(JText::_($header), $icon);
		
		foreach ($buttons AS $button)
		{
			$type = $button['type'];
			
			switch($type)
			{
				case 'divider':
				case 'spacer':
					JToolBarHelper::$type();
					break;
				case 'preferences':
					if ($this->canDo($assetName, $button['actions']))
					{
						JToolBarHelper::$type($assetName);
					}
					break;
				case 'deleteList':
				case 'delete':
					if ($this->canDo($assetName, $button['actions']))
					{
						$msg = $button['msg'];
						$task = $button['task'];
						$alt = $button['alt'];
					
						JToolBarHelper::$type($msg ,$task, $alt);
					}
					break;
					
				default:
					if ($this->canDo($assetName, $button['actions']))
					{
						$task = $button['task'];
						$alt = $button['alt'];
					
						JToolBarHelper::$type($task, $alt);
					}
					break;
			}
		}
		
	}
	
	/**
	 * Method to go through a list of actions to see if the user has access to the button
	 * @param string $assetName
	 * @param array $actions
	 * @return boolean
	 */
	private function canDo($assetName, $actions)
	{
		$user = JFactory::getUser();
		
		foreach ((array)$actions AS $action)
		{
			if (!$user->authorise($action, $assetName))
			{
				return false;
			}
		}
		
		return true;	
	}
	
}