<?php
/**
 * @version     0.0.1
 * @package     Babel-U-Lib
 * @copyright   Copyright (C) 2011. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author     Mathew Lenning - http://mathewlenning.com/
 */
// No direct access
defined('_JEXEC') or die;

class Babelu_libUtilitySidebar
{
	/**
	 * Array of classes to be added to the sidebar container
	 * @var array
	 */
	protected $classes = array();

	/**
	 * @var Babelu_libUtilitySubmenu
	*/
	protected $submenu;

	/**
	 * @var Babelu_libUtilityFilter
	 */
	protected $filters;

	public function __construct()
	{
		$this->submenu = new Babelu_libUtilitySubmenu();
		$this->filters = new Babelu_libUtilityFilter();
	}

	/**
	 * Method to add classes to the sidebar container
	 * @param string $class
	 */
	public function addClass($class)
	{
		array_push($this->classes, $this->cleanText($class));
	}

	/**
	 * Cleans text of all formatting and scripting code
	 *
	 * @param   string  $text  Text to clean
	 * @return  string  Cleaned text.
	 */
	protected  function cleanText($text)
	{
		$text = preg_replace("'<script[^>]*>.*?</script>'si", '', $text);
		$text = preg_replace('/<a\s+.*?href="([^"]+)"[^>]*>([^<]+)<\/a>/is', '\2 (\1)', $text);
		$text = preg_replace('/<!--.+?-->/', '', $text);
		$text = preg_replace('/{.+?}/', '', $text);
		$text = preg_replace('/&nbsp;/', ' ', $text);
		$text = preg_replace('/&amp;/', ' ', $text);
		$text = preg_replace('/&quot;/', ' ', $text);
		$text = strip_tags($text);
		$text = htmlspecialchars($text, ENT_COMPAT, 'UTF-8');

		return $text;
	}

	/**
	 * Method to add a menu item to sidebar.
	 *
	 * @param   string  $name    Name of the menu item.
	 * @param   string  $link    URL of the menu item.
	 * @param   bool    $active  True if the item is active, false otherwise.
	 * @see Babelu_libUtilitySubmenu::addEntry
	 *
	 */
	public function addEntry($name, $link = '', $isActive = false)
	{
		$this->submenu->addEntry($name, $link, $isActive);
	}

	/**
	 * Method to set the form action
	 * @param string $action
	 */
	public function setAction($action)
	{
		$this->filters->setAction($action);
	}

	/**
	 * Method to add a filter to the sidebar
	 * @param   string  $label      Label for the menu item.
	 * @param   string  $name       Name for the filter. Also used as id.
	 * @param   string  $options    Options for the select field.
	 * @param   bool    $noDefault  Don't the label as the empty option
	 * @see Babelu_libUtilityFilter::addFilter
	 */
	public function addFilter($label, $name, $options, $noDefault = false)
	{
		$this->filters->addFilter($label, $name, $options, $noDefault);
	}

	public function render()
	{
		$html = '<div id="sidebar">';
		$html .= '<div class="';
		foreach ($this->classes AS $class)
		{
			$html .=' '.$class;
		}
		$html .='">';
		$html .= $this->submenu->render();
		$html .= $this->filters->render();
		$html .='</div></div>';
		return $html;
	}
}

